/// <reference types="node" />
/// <reference types="node" />
import type { Readable } from 'stream';
import type { BinaryMetadata } from 'n8n-workflow';
import type { IBinaryDataConfig, IBinaryDataManager } from '../Interfaces';
export declare class BinaryDataFileSystem implements IBinaryDataManager {
    private storagePath;
    private binaryDataTTL;
    constructor(config: IBinaryDataConfig);
    init(startPurger?: boolean): Promise<void>;
    getFileSize(identifier: string): Promise<number>;
    copyBinaryFile(filePath: string, executionId: string): Promise<string>;
    storeBinaryMetadata(identifier: string, metadata: BinaryMetadata): Promise<void>;
    getBinaryMetadata(identifier: string): Promise<BinaryMetadata>;
    storeBinaryData(binaryData: Buffer | Readable, executionId: string): Promise<string>;
    getBinaryStream(identifier: string, chunkSize?: number): Readable;
    retrieveBinaryDataByIdentifier(identifier: string): Promise<Buffer>;
    getBinaryPath(identifier: string): string;
    getMetadataPath(identifier: string): string;
    markDataForDeletionByExecutionId(executionId: string): Promise<void>;
    deleteMarkedFiles(): Promise<void>;
    private deleteMarkedFilesByMeta;
    duplicateBinaryDataByIdentifier(binaryDataId: string, prefix: string): Promise<string>;
    deleteBinaryDataByExecutionIds(executionIds: string[]): Promise<string[]>;
    deleteBinaryDataByIdentifier(identifier: string): Promise<void>;
    private assertFolder;
    private generateFileName;
    private getBinaryDataMetaPath;
    private deleteFromLocalStorage;
    private copyFileToLocalStorage;
    private saveToLocalStorage;
    private retrieveFromLocalStorage;
    private resolveStoragePath;
}
